import React from 'react';
export declare type SelectorFn<Value, Selected> = (value: Value) => Selected;
export interface ContainerProviderProps<State = void, Value = void> {
    initialState?: State;
    children?: React.ReactNode | ((value: Value) => React.ReactNode);
}
/**
 * Create a container with `useHook`
 */
export declare function createContainer<Value, State = void>(useHook: (initialState?: State) => Value): {
    Context: React.Context<Value | null>;
    Provider: React.MemoExoticComponent<({ initialState, children }: ContainerProviderProps<State, Value>) => JSX.Element>;
    /**
     * Filter the required return value through the hook.
     * @param selector
     * @example
     * const count = useSelector(state => state.count)
     */
    useSelector: <Selected>(selector: SelectorFn<Value, Selected>) => Selected;
    /**
     * Filter the required values in the array, provided that `state` must be `object`.
     * @param selected
     * @example
     * const { count, setCount } = usePicker(['count', 'setCount'])
     */
    usePicker: <Selected_1 extends keyof Value>(selected: Selected_1[]) => Pick<Value, Selected_1>;
    /**
     * High-level component picker
     * @param Child
     * @param selected
     */
    withPicker: <T, Selected_2 extends keyof Value>(Child: React.FC<T & Pick<Value, Selected_2>>, selected: Selected_2[]) => React.FC<Pick<T, Exclude<keyof T, Selected_2>>>;
    /**
     * Provider wrapper.
     * @param Child
     */
    withProvider: <T_1>(Child: React.FC<T_1>, initialState?: State | undefined) => React.FC<T_1>;
};
/**
 * Shadow equal
 * @param origin
 * @param next
 */
declare function isShadowEqual(origin: unknown, next: unknown): boolean;
export { isShadowEqual };
export { default as useFunction } from './useFunction';
export { default as useMethods } from './useMethods';
