import React from 'react';

function useFunction(fn) {
  var _React$useRef = React.useRef({
    fn: fn,
    result: undefined
  }),
      current = _React$useRef.current;

  current.fn = fn;

  if (!current.result) {
    current.result = function () {
      var _current$fn;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return (_current$fn = current.fn).call.apply(_current$fn, [null].concat(args));
    };
  }

  return current.result;
}

function useMethods(methods) {
  var _React$useRef = React.useRef({
    methods: methods,
    func: undefined
  }),
      current = _React$useRef.current;

  current.methods = methods;

  if (!current.func) {
    var func = Object.create(null);
    Object.keys(methods).forEach(function (key) {
      func[key] = function () {
        var _current$methods$key;

        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return (_current$methods$key = current.methods[key]).call.apply(_current$methods$key, [current.methods].concat(args));
      };
    });
    current.func = func;
  }

  return current.func;
}

var isSSR = typeof window === 'undefined' || /ServerSideRendering/.test(window.navigator && window.navigator.userAgent);
var useIsomorphicLayoutEffect = isSSR ? React.useEffect : React.useLayoutEffect;
function createContainer(useHook) {
  var Context = React.createContext(null, function () {
    return 0;
  });
  var ListenerContext = React.createContext(new Set());
  var Provider = React.memo(function (_ref) {
    var initialState = _ref.initialState,
        children = _ref.children;
    var value = useHook(initialState);
    var listeners = React.useRef(new Set()).current;

    if (process.env.NODE_ENV !== 'production') {
      useIsomorphicLayoutEffect(function () {
        listeners.forEach(function (listener) {
          listener(value);
        });
      });
    } else {
      listeners.forEach(function (listener) {
        listener(value);
      });
    }

    return React.createElement(Context.Provider, {
      value: value
    }, React.createElement(ListenerContext.Provider, {
      value: listeners
    }, typeof children === 'function' ? children(value) : children));
  });

  function useSelector(selector) {
    var _React$useReducer = React.useReducer(function (c) {
      return c + 1;
    }, 0),
        forceUpdate = _React$useReducer[1];

    var value = React.useContext(Context);
    var listeners = React.useContext(ListenerContext);

    if (value === null) {
      throw new Error('The component must be wrapped by <Container.Provider />');
    }

    var selected = selector(value);
    var ref = React.useRef(null);
    useIsomorphicLayoutEffect(function () {
      ref.current = {
        selector: selector,
        value: value,
        selected: selected
      };
    });
    useIsomorphicLayoutEffect(function () {
      var callback = function callback(nextValue) {
        try {
          if (!ref.current) {
            return;
          }

          var refValue = ref.current;

          if (refValue.value === nextValue) {
            return;
          }

          var nextSelected = refValue.selector(nextValue);

          if (isShadowEqual(refValue.selected, nextSelected)) {
            return;
          }
        } catch (e) {}

        forceUpdate();
      };

      listeners.add(callback);
      return function () {
        listeners["delete"](callback);
      };
    }, []);
    return selected;
  }

  function usePicker(selected) {
    return useSelector(function (state) {
      return pick(state, selected);
    });
  }

  function withPicker(Child, selected) {
    return function (props) {
      var picked = usePicker(selected);
      return React.createElement(Child, Object.assign({}, picked, props));
    };
  }

  function withProvider(Child, initialState) {
    return function (props) {
      return React.createElement(Provider, {
        initialState: initialState
      }, React.createElement(Child, Object.assign({}, props)));
    };
  }

  return {
    Context: Context,
    Provider: Provider,
    useSelector: useSelector,
    usePicker: usePicker,
    withPicker: withPicker,
    withProvider: withProvider
  };
}

function pick(origin, keys) {
  var empty = {};

  if (!origin) {
    return empty;
  }

  return Object.assign.apply(Object, [empty].concat(keys.map(function (key) {
    var _ref2;

    return _ref2 = {}, _ref2[key] = origin[key], _ref2;
  })));
}

function isShadowEqual(origin, next) {
  if (Object.is(origin, next)) {
    return true;
  }

  if (origin && typeof origin === 'object' && next && typeof next === 'object') {
    if ([].concat(Object.keys(origin), Object.keys(next)).every(function (k) {
      return origin[k] === next[k] && origin.hasOwnProperty(k) && next.hasOwnProperty(k);
    })) {
      return true;
    }
  }

  return false;
}

export { createContainer, isShadowEqual, useFunction, useMethods };
//# sourceMappingURL=index.modern.js.map
